%
%
%  Spots Create Channel Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Spots Functions">
%        <Item name="Create Channel from Spots" icon="Matlab">
%          <Command>MatlabXT::XTSpotsCreateChannel(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Create Channel" icon="Matlab">
%            <Command>MatlabXT::XTSpotsCreateChannel(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
% !!! Additional informations:
% A new channel is added to the dataset
% Original dataset size is maintained
%

function XTSpotsCreateChannel(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% the user has to create a scene with some spots
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
    msgbox('Please create some Spots in the Surpass scene!');
    return;
end
vImarisApplication.DataSetPushUndo('Create Channel from Spots');

% get the spots
vSpots = vImarisApplication.GetFactory.ToSpots(vImarisApplication.GetSurpassSelection);

% search the spots if not previously selected
if ~vImarisApplication.GetFactory.IsSpots(vSpots)        
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vImarisApplication.GetFactory.IsSpots(vDataItem)
            vSpots = vImarisApplication.GetFactory.ToSpots(vDataItem);
            break;
        end
    end
    % did we find the spots?
    if isequal(vSpots, [])
        msgbox('Please create some spots!');
        return;
    end
end

vInterpolate = questdlg('Interpolate intensity along spots radius?', ...
  'XTSpotsCreateChannel',  'Yes', 'No', 'Cancel', 'Yes');
if isequal(vInterpolate, 'Cancel'), return, end
vInterpolate = isequal(vInterpolate, 'Yes');

% get the spots coordinates
vSpotsXYZ = vSpots.GetPositionsXYZ;
vSpotsRadius = vSpots.GetRadiiXYZ;
% Use the mean of the radius on the three dimensions
%vSpotsRadius = mean(vSpotsRadiusXYZ,2);
vSpotsTime = vSpots.GetIndicesT;

% add vChannel to current dataset
[vDataSet, vData, vMin, vMax, vType] = InitNewChannel(vImarisApplication);

vMaxTime = vDataSet.GetSizeT - 1;
vChannel = vDataSet.GetSizeC - 1;

vProgressDisplay = waitbar(0,'Creating DataSet');
for vTime = 0:vMaxTime
  if vTime > 0
    vData(:) = 0;
  end
  
  vSpotsIndex = find(vSpotsTime == vTime)';
  for vIndex = vSpotsIndex
    vPos = vSpotsXYZ(vIndex, :);
    vRadius = vSpotsRadius(vIndex,:);
    vData = DrawSphere(vData, vPos, vRadius, vMin, vMax, vType, vInterpolate);
  end
  if ~isempty(vSpotsIndex)
    if strcmp(vDataSet.GetType, 'eTypeUInt8')
      vDataSet.SetDataVolumeBytes(vData, vChannel, vTime);
    elseif strcmp(vDataSet.GetType, 'eTypeUInt16')
      vDataSet.SetDataVolumeShorts(vData, vChannel, vTime);
    else
      vDataSet.SetDataVolumeFloats(vData, vChannel, vTime);
    end
  end
  
  waitbar(vTime/vMaxTime)
end
close(vProgressDisplay);

vRGBA = vSpots.GetColorRGBA;
vDataSet.SetChannelColorRGBA(vChannel, vRGBA);
vDataSet.SetChannelName(vChannel, vSpots.GetName);
vImarisApplication.SetDataSet(vDataSet);


function aData = DrawSphere(aData, aPos, aRad, aMin, aMax, aType, aInterpolate)
vSize = [size(aData, 1), size(aData, 2), size(aData, 3)];
aPos = (aPos - aMin) ./ (aMax - aMin) .* vSize + 0.5;
aRad = aRad ./ (aMax - aMin) .* vSize;
vPosMin = round(max(aPos - aRad, 1));
vPosMax = round(min(aPos + aRad, vSize));
vPosX = vPosMin(1):vPosMax(1);
vPosY = vPosMin(2):vPosMax(2);
vPosZ = vPosMin(3):vPosMax(3);
[vX, vY, vZ] = ndgrid(vPosX, vPosY, vPosZ);
vDist = ((vX - aPos(1))/aRad(1)).^2 + ((vY - aPos(2))/aRad(2)).^2 + ...
     ((vZ - aPos(3))/aRad(3)).^2;
vInside = vDist < 1;
vCube = aData(vPosX, vPosY, vPosZ);
if aInterpolate
  vCube(vInside) = max(vCube(vInside), FixType(255*(1-vDist(vInside)), aType));
else
  vCube(vInside) = 255;
end
aData(vPosX, vPosY, vPosZ) = vCube;


function aData = FixType(aData, aType)
if strcmp(aType, 'eTypeUInt8')
  aData = uint8(aData);
elseif strcmp(aType, 'eTypeUInt16')
  aData = uint16(aData);
else
  aData = single(aData);
end


function [aDataSet, aData, aMin, aMax, aType] = InitNewChannel(aImaris)
aDataSet = aImaris.GetDataSet.Clone;
aMin = [aDataSet.GetExtendMinX, aDataSet.GetExtendMinY, aDataSet.GetExtendMinZ];
aMax = [aDataSet.GetExtendMaxX, aDataSet.GetExtendMaxY, aDataSet.GetExtendMaxZ];
vSize = [aDataSet.GetSizeX, aDataSet.GetSizeY, aDataSet.GetSizeZ];
if strcmp(aDataSet.GetType, 'eTypeUInt8')
  aData = zeros(vSize, 'uint8');
elseif strcmp(aDataSet.GetType, 'eTypeUInt16')
  aData = zeros(vSize, 'uint16');
else
  aData = zeros(vSize, 'single');
end
aDataSet.SetSizeC(aDataSet.GetSizeC + 1);
aType = aDataSet.GetType;
